@@ -22,7 +22,6 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
-#include <signal.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <sys/wait.h>
@@ -33,26 +32,30 @@
 #include <ctype.h>
 #include <assert.h>
 
-#include <ncurses.h>
 #include <term.h>
 
 extern "C" {
-#include "config.h"
-#include "dpkg.h"
-#include "dpkg-db.h"
-#include "version.h"
-#include "myopt.h"
+#include <config.h>
+#include <dpkg.h>
+#include <dpkg-db.h>
+#include <version.h>
+#include <myopt.h>
 }
 #include "dselect.h"
 #include "bindings.h"
 #include "pkglist.h"
 
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+
 const char thisname[]= DSELECT;
-const char printforhelp[]= "Type " DPKG " --help for help.";
+const char printforhelp[]= N_("Type dselect --help for help.");
 
 modstatdb_rw readwrite;
 const char *admindir= ADMINDIR;
 FILE *debug;
+int expertmode = 0;
 
 static keybindings packagelistbindings(packagelist_kinterps,packagelist_korgbindings);
 
@@ -63,36 +66,38 @@ struct menuentry {
 };
 
 static const menuentry menuentries[]= {
-  { "access",  "Choose the access method to use.",                &urq_setup   },
-  { "update",  "Update list of available packages, if possible.", &urq_update  },
-  { "select",  "Request which packages you want on your system.", &urq_list    },
-  { "install", "Install and upgrade wanted packages.",            &urq_install },
-  { "config",  "Configure any packages that are unconfigured.",   &urq_config  },
-  { "remove",  "Remove unwanted software.",                       &urq_remove  },
-  { "quit",    "Quit dselect.",                                   &urq_quit    },
-  { "menu",     0,                                                &urq_menu    },
-  {  0                                                                         }
+  { N_("access"),  N_("Choose the access method to use."),                &urq_setup   },
+  { N_("update"),  N_("Update list of available packages, if possible."), &urq_update  },
+  { N_("select"),  N_("Request which packages you want on your system."), &urq_list    },
+  { N_("install"), N_("Install and upgrade wanted packages."),            &urq_install },
+  { N_("config"),  N_("Configure any packages that are unconfigured."),   &urq_config  },
+  { N_("remove"),  N_("Remove unwanted software."),                       &urq_remove  },
+  { N_("quit"),    N_("Quit dselect."),                                   &urq_quit    },
+  { N_("menu"),    0,                                                     &urq_menu    },
+  { 0                                                                              }
 };
 
 static const char programdesc[]=
-      "Debian Linux `" DSELECT "' package handling frontend.";
+      N_("Debian GNU/Linux `%s' package handling frontend.");
 
-static const char copyrightstring[]=
-      "Version " DPKG_VERSION_ARCH ".  Copyright (C) 1994-1996 Ian Jackson.   This is\n"
+static const char copyrightstring[]= N_(
+      "Version %s.  Copyright (C) 1994-1996 Ian Jackson.   This is\n"
       "free software; see the GNU General Public Licence version 2 or later for\n"
-      "copying conditions.  There is NO warranty.  See dselect --licence for details.\n";
+      "copying conditions.  There is NO warranty.  See dselect --licence for details.\n");
 
 static void printversion(void) {
-  if (fprintf(stdout,"%s\n%s",programdesc,copyrightstring) == EOF) werr("stdout");
+  if (fprintf(stdout,gettext(programdesc),DSELECT) == EOF) werr("stdout");
+  if (fprintf(stdout,"\n") == EOF) werr("stdout");
+  if (fprintf(stdout,gettext(copyrightstring), DPKG_VERSION_ARCH) == EOF) werr("stdout");
 }
 
 static void usage(void) {
-  if (!fputs
-      ("Usage: dselect [options]\n"
+  if (!fputs(
+     _("Usage: dselect [options]\n"
        "       dselect [options] action ...\n"
        "Options:  --admindir <directory>  (default is /var/lib/dpkg)\n"
-       "          --help  --version  --licence   --debug <file> | -D<file> | -D\n"
-       "Actions:  access update select install config remove quit menu\n",
+       "          --help  --version  --licence  --expert  --debug <file> | -D<file>\n"
+       "Actions:  access update select install config remove quit menu\n"),
        stdout)) werr("stdout");
 }
 
@@ -108,15 +113,20 @@ extern "C" {
 
   static void setdebug(const struct cmdinfo*, const char *v) {
     debug= fopen(v,"a");
-    if (!debug) ohshite("couldn't open debug file `%.255s'\n",v);
+    if (!debug) ohshite(_("couldn't open debug file `%.255s'\n"),v);
     setvbuf(debug,0,_IONBF,0);
   }
 
+  static void setexpert() {
+    expertmode = 1;
+  }
+
 } /* End of extern "C" */
 
 static const struct cmdinfo cmdinfos[]= {
   { "admindir",   0,   1,  0,  &admindir,  0                      },
   { "debug",     'D',  1,  0,  0,          setdebug               },
+  { "expert",    'E',  0,  0,  0,          setexpert              },
   { "help",      'h',  0,  0,  0,          helponly               },
   { "version",    0,   0,  0,  0,          versiononly            },
   { "licence",    0,   0,  0,  0,          showcopyright          }, /* UK spelling */
@@ -134,12 +144,12 @@ void curseson() {
     if (!cup || !smso) {
       endwin();
       if (!cup)
-        fputs("Terminal does not appear to support cursor addressing.\n",stderr);
+        fputs(_("Terminal does not appear to support cursor addressing.\n"),stderr);
       if (!smso)
-        fputs("Terminal does not appear to support highlighting.\n",stderr);
-      fputs("Set your TERM variable correctly, use a better terminal,\n"
-            "or make do with the per-package management tool " DPKG ".\n",stderr);
-      ohshit("terminal lacks necessary features, giving up");
+        fputs(_("Terminal does not appear to support highlighting.\n"),stderr);
+      fputs(_("Set your TERM variable correctly, use a better terminal,\n"
+            "or make do with the per-package management tool " DPKG ".\n"),stderr);
+      ohshit(_("terminal lacks necessary features, giving up"));
     }
   }
   cursesareon= 1;
@@ -183,10 +193,11 @@ urqresult urq_list(void) {
 void dme(int i, int so) {
   char buf[120];
   const menuentry *me= &menuentries[i];
+  const char* option = gettext(me->option);
   sprintf(buf," %c %d. [%c]%-10.10s %-80.80s ",
           so ? '*' : ' ', i,
-          toupper(me->option[0]), me->option+1,
-          me->menuent);
+          toupper(option[0]), option+1,
+          gettext(me->menuent));
   
   int y,x;
   getmaxyx(stdscr,y,x);
@@ -197,6 +208,10 @@ void dme(int i, int so) {
 }
 
 int refreshmenu(void) {
+  char buf[2048];
+  static int l,lockfd;
+  static char *lockfile;
+
   curseson(); cbreak(); noecho(); nonl(); keypad(stdscr,TRUE);
 
   int y,x;
@@ -204,7 +219,8 @@ int refreshmenu(void) {
 
   clear();
   attrset(A_BOLD);
-  mvaddnstr(0,0, programdesc,x-1);
+  sprintf(buf,gettext(programdesc),DSELECT);
+  mvaddnstr(0,0,buf,x-1);
 
   attrset(A_NORMAL);
   const struct menuentry *mep; int i;
@@ -212,12 +228,22 @@ int refreshmenu(void) {
     dme(i,0);
 
   attrset(A_BOLD);
-  addstr("\n\n"
-         "Use ^P and ^N, cursor keys, initial letters, or digits to select;\n"
-         "Press ENTER to confirm selection.   ^L to redraw screen.\n\n");
+  addstr(_("\n\n"
+         "Move around with ^P and ^N, cursor keys, initial letters, or digits;\n"
+         "Press <enter> to confirm selection.   ^L redraws screen.\n\n"));
 
   attrset(A_NORMAL);
-  addstr(copyrightstring);
+  sprintf(buf,gettext(copyrightstring),DPKG_VERSION_ARCH);
+  addstr(buf);
+
+  l = strlen(admindir);
+  lockfile = new char[l+sizeof(LOCKFILE)+2];
+  strcpy(lockfile,admindir);
+  strcpy(lockfile+l, "/" LOCKFILE);
+  lockfd = open(lockfile, O_RDWR|O_CREAT|O_TRUNC, 0660);
+  if (errno == EACCES || errno == EPERM)
+    addstr(_("\n\n"
+             "Read-only access: only preview of selections is available!"));
 
   return i;
 }
@@ -230,7 +256,7 @@ urqresult urq_menu(void) {
   dme(0,1);
   for (;;) {
     refresh();
-    c= getch();  if (c==ERR) ohshite("failed to getch in main menu");
+    c= getch();  if (c==ERR) ohshite(_("failed to getch in main menu"));
     if (c==C('n') || c==KEY_DOWN || c==' ') {
       dme(cursor,0); cursor++; cursor %= entries; dme(cursor,1);
     } else if (c==C('p') || c==KEY_UP || c==C('h') ||
@@ -260,7 +286,7 @@ urqresult urq_menu(void) {
       }
     } else if (isalpha(c)) {
       c= tolower(c);
-      for (i=0; i<entries && menuentries[i].option[0] != c; i++);
+      for (i=0; i<entries && gettext(menuentries[i].option)[0] != c; i++);
       if (i < entries) {
         dme(cursor,0); cursor=i; dme(cursor,1);
       } else {
@@ -280,6 +306,10 @@ urqresult urq_quit(void) {
 int main(int, const char *const *argv) {
   jmp_buf ejbuf;
 
+  setlocale(LC_ALL, "");
+  bindtextdomain(PACKAGE, LOCALEDIR);
+  textdomain(PACKAGE);
+
   if (setjmp(ejbuf)) { /* expect warning about possible clobbering of argv */
     cursesoff();
     error_unwind(ehflag_bombout); exit(2);
@@ -293,7 +323,7 @@ int main(int, const char *const *argv) {
     while ((a= *argv++) != 0) {
       const menuentry *me;
       for (me= menuentries; me->option && strcmp(me->option,a); me++);
-      if (!me->option) badusage("unknown action string `%.50s'",a);
+      if (!me->option) badusage(_("unknown action string `%.50s'"),a);
       me->fn();
     }
   } else {
