@@ -28,8 +28,7 @@
 void showcopyright(const struct cmdinfo *c, const char *v) {
   int fd;
   fd= open(COPYINGFILE,O_RDONLY);
-  if (fd < 0) ohshite(_("cannot open GPL file /usr/doc/dpkg/copyright"));
-  m_dup2(fd,0);
-  execlp(CAT,CAT,"-",(char*)0);
-  ohshite(_("unable to exec cat for displaying GPL file"));
+  if (fd < 0) ohshite(_("cannot open GPL file " COPYINGFILE));
+  do_fd_copy(fd, 1, _("showcopyright"));
+  exit(0);
 }
