@@ -25,13 +25,17 @@
 #include <errno.h>
 #include <sys/stat.h>
 
-#include "config.h"
-#include "dpkg.h"
-#include "dpkg-db.h"
+#include <config.h>
+#include <dpkg.h>
+#include <dpkg-db.h>
 
 #include "filesdb.h"
 #include "main.h"
 
+#ifdef HAVE_SYS_SYSINFO_H
+#include <sys/sysinfo.h>
+#endif
+
 /*** Generic data structures and routines ***/
 
 static int allpackagesdone= 0;
@@ -47,17 +51,20 @@ void note_must_reread_files_inpackage(struct pkginfo *pkg) {
 
 static int saidread=0;
 
+ /* load the list of files in this package into memory, or update the
+  * list if it is there but stale
+  */
 void ensure_packagefiles_available(struct pkginfo *pkg) {
   static struct varbuf fnvb;
-  static char stdiobuf[8192];
   
   FILE *file;
   const char *filelistfile;
   struct fileinlist **lendp, *newent, *current;
   struct filepackages *packageslump;
-  int search, findlast, putat, l;
-  char *thefilename;
-  char linebuf[1024];
+  int search, findlast, putat;
+  struct stat stat_buf;
+  char *loaded_list, *loaded_list_end, *thisline, *nextline, *ptr;
+  ssize_t bytes, readden;
 
   if (pkg->clientdata && pkg->clientdata->fileslistvalid) return;
   ensure_package_clientdata(pkg);
@@ -113,13 +120,13 @@ void ensure_packagefiles_available(struct pkginfo *pkg) {
 
   if (!file) {
     if (errno != ENOENT)
-      ohshite("unable to open files list file for package `%.250s'",pkg->name);
+      ohshite(_("unable to open files list file for package `%.250s'"),pkg->name);
     onerr_abort--;
     if (pkg->status != stat_configfiles) {
       if (saidread == 1) putc('\n',stderr);
       fprintf(stderr,
-              DPKG ": serious warning: files list file for package `%.250s' missing,"
-              " assuming package has no files currently installed.\n", pkg->name);
+              _("dpkg: serious warning: files list file for package `%.250s' missing,"
+              " assuming package has no files currently installed.\n"), pkg->name);
     }
     pkg->clientdata->files= 0;
     pkg->clientdata->fileslistvalid= 1;
@@ -128,49 +135,49 @@ void ensure_packagefiles_available(struct pkginfo *pkg) {
 
   push_cleanup(cu_closefile,ehflag_bombout, 0,0, 1,(void*)file);
   
-  if (setvbuf(file,stdiobuf,_IOFBF,sizeof(stdiobuf)))
-    ohshite("unable to set buffering on `%.250s'",filelistfile);
+   if(fstat(fileno(file), &stat_buf))
+     ohshite("unable to stat files list file for package `%.250s'",pkg->name);
+   loaded_list = nfmalloc(stat_buf.st_size);
+   loaded_list_end = loaded_list + stat_buf.st_size;
+   /* stdio is an extra copy, hence we use read() */
+   readden = 0;			/* write->written, read->readden */
+   while (readden < stat_buf.st_size) {
+     bytes = read(fileno(file),
+ 		 loaded_list + readden, stat_buf.st_size - readden);
+     if (bytes < 0) {
+       if (errno == EINTR) continue;
+       ohshite("unable to read files list for package `%.250s'",pkg->name);
+     }
+     if (!bytes)
+       ohshit("unexpected end of file in files list for package `%.250s'",pkg->name);
+     readden += bytes;
+   }
   
   lendp= &pkg->clientdata->files;
-  varbufreset(&fnvb);
-  while (fgets(linebuf,sizeof(linebuf),file)) {
-    /* This is a very important loop, and it is therefore rather messy.
-     * We break the varbuf abstraction even more than usual, and we
-     * avoid copying where possible.
-     */
-    l= strlen(linebuf);
-    if (l == 0) ohshit("fgets gave an empty null-terminated string from `%.250s'",
-                       filelistfile);
-    l--;
-    if (linebuf[l] != '\n') {
-      varbufaddstr(&fnvb,linebuf);
-      continue;
-    } else if (!fnvb.used && l>0 && linebuf[l-1] != '/') { /* fast path */
-      linebuf[l]= 0;
-      thefilename= linebuf;
-    } else {
-      if (l>0 && linebuf[l-1] == '/') l--; /* strip trailing slashes */
-      linebuf[l]= 0;
-      varbufaddstr(&fnvb,linebuf);
-      varbufaddc(&fnvb,0);
-      fnvb.used= 0;
-      thefilename= fnvb.buf;
-    }
-    if (!*thefilename)
-      ohshit("files list file for package `%.250s' contains empty filename",pkg->name);
+  thisline = loaded_list;
+  while (thisline < loaded_list_end) {
+    if (!(ptr = memchr(thisline, '\n', loaded_list_end - thisline))) 
+      ohshit("files list file for package `%.250s' is missing final newline",pkg->name);
+    /* where to start next time around */
+    nextline = ptr + 1;
+    /* strip trailing "/" */
+    if (ptr > thisline && ptr[-1] == '/') ptr--;
+    /* add the file to the list */
+    if (ptr == thisline)
+      ohshit(_("files list file for package `%.250s' contains empty filename"),pkg->name);
+    *ptr = 0;
     newent= nfmalloc(sizeof(struct fileinlist));
-    newent->namenode= findnamenode(thefilename);
+    newent->namenode= findnamenode(thisline, fnn_nocopy);
     newent->next= 0;
     *lendp= newent;
     lendp= &newent->next;
+    thisline = nextline;
   }
-  if (ferror(file))
-    ohshite("error reading files list file for package `%.250s'",pkg->name);
   pop_cleanup(ehflag_normaltidy); /* file= fopen() */
   if (fclose(file))
-    ohshite("error closing files list file for package `%.250s'",pkg->name);
+    ohshite(_("error closing files list file for package `%.250s'"),pkg->name);
   if (fnvb.used)
-    ohshit("files list file for package `%.250s' is truncated",pkg->name);
+    ohshit(_("files list file for package `%.250s' is truncated"),pkg->name);
 
   onerr_abort--;
 
@@ -201,7 +208,7 @@ void ensure_allinstfiles_available(void) {
   if (allpackagesdone) return;
   if (saidread<2) {
     saidread=1;
-    printf(f_largemem>0 ? "(Reading database ... " : "(Scanning database ... ");
+    printf(f_largemem>0 ? _("(Reading database ... ") : _("(Scanning database ... "));
   }
   it= iterpkgstart();
   while ((pkg= iterpkgnext(it)) != 0) ensure_packagefiles_available(pkg);
@@ -209,7 +216,7 @@ void ensure_allinstfiles_available(void) {
   allpackagesdone= 1;
 
   if (saidread==1) {
-    printf("%d files and directories currently installed.)\n",nfiles);
+    printf(_("%d files and directories currently installed.)\n"),nfiles);
     saidread=2;
   }
 }
@@ -240,7 +247,7 @@ void write_filelist_except(struct pkginfo *pkg, struct fileinlist *list, int lea
   
   file= fopen(newvb.buf,"w+");
   if (!file)
-    ohshite("unable to create updated files list file for package %s",pkg->name);
+    ohshite(_("unable to create updated files list file for package %s"),pkg->name);
   push_cleanup(cu_closefile,ehflag_bombout, 0,0, 1,(void*)file);
   while (list) {
     if (!(leaveout && (list->namenode->flags & fnnf_elide_other_lists))) {
@@ -250,16 +257,16 @@ void write_filelist_except(struct pkginfo *pkg, struct fileinlist *list, int lea
     list= list->next;
   }
   if (ferror(file))
-    ohshite("failed to write to updated files list file for package %s",pkg->name);
+    ohshite(_("failed to write to updated files list file for package %s"),pkg->name);
   if (fflush(file))
-    ohshite("failed to flush updated files list file for package %s",pkg->name);
+    ohshite(_("failed to flush updated files list file for package %s"),pkg->name);
   if (fsync(fileno(file)))
-    ohshite("failed to sync updated files list file for package %s",pkg->name);
+    ohshite(_("failed to sync updated files list file for package %s"),pkg->name);
   pop_cleanup(ehflag_normaltidy); /* file= fopen() */
   if (fclose(file))
-    ohshite("failed to close updated files list file for package %s",pkg->name);
+    ohshite(_("failed to close updated files list file for package %s"),pkg->name);
   if (rename(newvb.buf,vb.buf))
-    ohshite("failed to install updated files list file for package %s",pkg->name);
+    ohshite(_("failed to install updated files list file for package %s"),pkg->name);
 
   note_must_reread_files_inpackage(pkg);
 }
@@ -323,13 +330,13 @@ void ensure_diversions(void) {
   
   file= fopen(vb.buf,"r");
   if (!file) {
-    if (errno != ENOENT) ohshite("failed to open diversions file");
+    if (errno != ENOENT) ohshite(_("failed to open diversions file"));
     if (!diversionsfile) { onerr_abort--; return; }
   } else if (diversionsfile) {
     if (fstat(fileno(diversionsfile),&stab1))
-      ohshite("failed to fstat previous diversions file");
+      ohshite(_("failed to fstat previous diversions file"));
     if (fstat(fileno(file),&stab2))
-      ohshite("failed to fstat diversions file");
+      ohshite(_("failed to fstat diversions file"));
     if (stab1.st_dev == stab2.st_dev && stab1.st_ino == stab2.st_ino) {
       fclose(file); onerr_abort--; return;
     }
@@ -349,33 +356,37 @@ void ensure_diversions(void) {
     oialtname= nfmalloc(sizeof(struct diversion));
 
     l= strlen(linebuf);
-    if (l == 0) ohshit("fgets gave an empty string from diversions [i]");
-    if (linebuf[--l] != '\n') ohshit("diversions file has too-long line or EOF [i]");
+    if (l == 0) ohshit(_("fgets gave an empty string from diversions [i]"));
+    if (linebuf[--l] != '\n') ohshit(_("diversions file has too-long line or EOF [i]"));
     linebuf[l]= 0;
-    oialtname->camefrom= findnamenode(linebuf);
-    
-    if (!fgets(linebuf,sizeof(linebuf),file))
-      if (ferror(file)) ohshite("read error in diversions [ii]");
-      else ohshit("unexpected EOF in diversions [ii]");
+    oialtname->camefrom= findnamenode(linebuf, 0);
+    oialtname->useinstead= 0;    
+
+    if (!fgets(linebuf,sizeof(linebuf),file)) {
+      if (ferror(file)) ohshite(_("read error in diversions [ii]"));
+      else ohshit(_("unexpected EOF in diversions [ii]"));
+    } 
     l= strlen(linebuf);
-    if (l == 0) ohshit("fgets gave an empty string from diversions [ii]");
-    if (linebuf[--l] != '\n') ohshit("diversions file has too-long line or EOF [ii]");
+    if (l == 0) ohshit(_("fgets gave an empty string from diversions [ii]"));
+    if (linebuf[--l] != '\n') ohshit(_("diversions file has too-long line or EOF [ii]"));
     linebuf[l]= 0;
-    oicontest->useinstead= findnamenode(linebuf);
+    oicontest->useinstead= findnamenode(linebuf, 0);
+    oicontest->camefrom= 0;
     
-    if (!fgets(linebuf,sizeof(linebuf),file))
-      if (ferror(file)) ohshite("read error in diversions [iii]");
-      else ohshit("unexpected EOF in diversions [iii]");
+    if (!fgets(linebuf,sizeof(linebuf),file)) {
+      if (ferror(file)) ohshite(_("read error in diversions [iii]"));
+      else ohshit(_("unexpected EOF in diversions [iii]"));
+    }
     l= strlen(linebuf);
-    if (l == 0) ohshit("fgets gave an empty string from diversions [iii]");
-    if (linebuf[--l] != '\n') ohshit("diversions file has too-long line or EOF [ii]");
+    if (l == 0) ohshit(_("fgets gave an empty string from diversions [iii]"));
+    if (linebuf[--l] != '\n') ohshit(_("diversions file has too-long line or EOF [ii]"));
     linebuf[l]= 0;
 
     oicontest->pkg= oialtname->pkg=
       strcmp(linebuf,":") ? findpackage(linebuf) : 0;
 
     if (oialtname->camefrom->divert || oicontest->useinstead->divert)
-      ohshit("conflicting diversions involving `%.250s' or `%.250s'",
+      ohshit(_("conflicting diversions involving `%.250s' or `%.250s'"),
              oialtname->camefrom->name, oicontest->useinstead->name);
 
     oialtname->camefrom->divert= oicontest;
@@ -384,7 +395,7 @@ void ensure_diversions(void) {
     oicontest->next= diversions;
     diversions= oicontest;
   }
-  if (ferror(file)) ohshite("read error in diversions [i]");
+  if (ferror(file)) ohshite(_("read error in diversions [i]"));
 
   diversionsfile= file;
   onerr_abort--;
@@ -416,6 +427,26 @@ static struct filenamenode *bins[BINS];
 
 /*** Data structures for low-memory-footprint in-core files database ***/
 
+/* the idea is that you have a tree structure in memory which has the
+   same structure as the names themselves.
+
+   Each node in the tree gets an fdirnode.  This may have a
+   filenamenode attached to it (if there is really a filename
+   corresponding to the path down the tree to get here) and an
+   fdirents (if there is anything below this point.)
+
+   The fdirents structure lists the entries in a directory.  If there
+   is only 1 node below us then there's just one fdirents with a
+   single entry; if there are more then then next one (as defined by
+   the 'more' field) contains two entries; the next four; etc.
+
+   This doubling effect is enforced by findnamenow_low() rather than
+   by a count field in the structure.
+
+   1999-07-26 RJK
+   
+*/
+
 struct fdirents {
   struct fdirents *more;
   struct { const char *component; struct fdirnode *go; } entries[1];
@@ -483,26 +514,31 @@ void iterfileend(struct fileiterator *i) {
   free(i);
 }
 
+static int autodetect_largemem(void) {
+#if defined(HAVE_SYSINFO) && defined (MEMINFO_IN_SYSINFO)
+  struct sysinfo info;
+  if (sysinfo(&info)) return 0;
+  if (info.freeram + (info.sharedram>>2) + (info.bufferram>>2) < 24576*1024 &&
+      info.totalram < 24576*1024)
+    return 0;
+#endif
+  return 1;
+}
+
 void filesdbinit(void) {
   struct filenamenode *fnn;
-  struct sysinfo info;
   int i;
 
-  if (!f_largemem) {
-    f_largemem= -1;
-    if (!sysinfo(&info)) {
-      if (info.freeram + (info.sharedram>>2) + (info.bufferram>>2) >= 4096*1024 ||
-          info.totalram >= 6144)
-        f_largemem= 1;
-    }
-  }
-  
+  if (!f_largemem)
+    f_largemem= autodetect_largemem() ? 1 : -1;
+
   switch (f_largemem) {
   case 1:
     for (i=0; i<BINS; i++)
       for (fnn= bins[i]; fnn; fnn= fnn->next) {
         fnn->flags= 0;
         fnn->oldhash= 0;
+        fnn->filestat= 0;
       }
     break;
   case -1:
@@ -511,6 +547,7 @@ void filesdbinit(void) {
          fnn= fnn->next) {
       fnn->flags= 0;
       fnn->oldhash= 0;
+      fnn->filestat= 0;
     }
     break;
   default:
@@ -518,15 +555,17 @@ void filesdbinit(void) {
   }    
 }
 
-static struct filenamenode *findnamenode_high(const char *name);
-static struct filenamenode *findnamenode_low(const char *name);
+static struct filenamenode *findnamenode_high(const char *name,
+					      enum fnnflags flags);
+static struct filenamenode *findnamenode_low(const char *name,
+					     enum fnnflags flags);
   
-struct filenamenode *findnamenode(const char *name) {
+struct filenamenode *findnamenode(const char *name, enum fnnflags flags) {
   switch (f_largemem) {
   case 1:
-    return findnamenode_high(name);
+    return findnamenode_high(name, flags);
   case -1:
-    return findnamenode_low(name);
+    return findnamenode_low(name, flags);
   default:
     internerr("findnamenode no f_largemem");
   }
@@ -534,14 +573,16 @@ struct filenamenode *findnamenode(const char *name) {
 
 /*** Code for low-memory-footprint in-core files database ***/
   
-static struct filenamenode *findnamenode_low(const char *name) {
+static struct filenamenode *findnamenode_low(const char *name,
+					     enum fnnflags flags) {
   struct fdirnode *traverse;
   struct fdirents *ents, **addto;
   const char *nameleft, *slash;
   char *p;
   struct filenamesblock *newblock;
   int n, i, nentrieshere, alloc;
-
+  const char *orig_name = name;
+  
   /* We skip initial slashes and ./ pairs, and add our own single leading slash. */
   name= skip_slash_dotslash(name);
 
@@ -596,20 +637,25 @@ static struct filenamenode *findnamenode_low(const char *name) {
   traverse->here->packages= 0;
   traverse->here->flags= 0;
   traverse->here->divert= 0;
+  traverse->here->filestat= 0;
 
-  n= strlen(name)+2;
-  if (namesarealeft < n) {
-    newblock= m_malloc(sizeof(struct filenamesblock));
-    alloc= 256*1024;
-    if (alloc<n) alloc= n;
-    newblock->data= m_malloc(alloc);
-    newblock->next= namesarea;
-    namesarea= newblock;
-    namesarealeft= alloc;
+  if((flags & fnn_nocopy) && name > orig_name && name[-1] == '/') {
+    traverse->here->name = (char *)name - 1;
+  } else {
+    n= strlen(name)+2;
+    if (namesarealeft < n) {
+      newblock= m_malloc(sizeof(struct filenamesblock));
+      alloc= 256*1024;
+      if (alloc<n) alloc= n;
+      newblock->data= m_malloc(alloc);
+      newblock->next= namesarea;
+      namesarea= newblock;
+      namesarealeft= alloc;
+    }
+    namesarealeft-= n;
+    p= namesarea->data+namesarealeft;
+    traverse->here->name= p; *p++= '/'; strcpy(p,name);
   }
-  namesarealeft-= n;
-  p= namesarea->data+namesarealeft;
-  traverse->here->name= p; *p++= '/'; strcpy(p,name);
 
   traverse->here->next= allfiles;
   allfiles= traverse->here;
@@ -625,8 +671,10 @@ static int hash(const char *name) {
   return v;
 }
 
-struct filenamenode *findnamenode_high(const char *name) {
+struct filenamenode *findnamenode_high(const char *name,
+					     enum fnnflags flags) {
   struct filenamenode **pointerp, *newnode;
+  const char *orig_name = name;
 
   /* We skip initial slashes and ./ pairs, and add our own single leading slash. */
   name= skip_slash_dotslash(name);
@@ -641,11 +689,16 @@ struct filenamenode *findnamenode_high(const char *name) {
 
   newnode= nfmalloc(sizeof(struct filenamenode));
   newnode->packages= 0;
-  newnode->name= nfmalloc(strlen(name)+2);
-  newnode->name[0]= '/'; strcpy(newnode->name+1,name);
+  if((flags & fnn_nocopy) && name > orig_name && name[-1] == '/')
+    newnode->name = (char *)name - 1;
+  else {
+    newnode->name= nfmalloc(strlen(name)+2);
+    newnode->name[0]= '/'; strcpy(newnode->name+1,name);
+  }
   newnode->flags= 0;
   newnode->next= 0;
   newnode->divert= 0;
+  newnode->filestat= 0;
   *pointerp= newnode;
   nfiles++;
 
