@@ -19,6 +19,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#include <errno.h>
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
@@ -33,11 +34,11 @@
 #include <sys/types.h>
 #include <sys/wait.h>
 
-#include "config.h"
-#include "dpkg.h"
-#include "dpkg-db.h"
-#include "myopt.h"
-#include "tarfn.h"
+#include <config.h>
+#include <dpkg.h>
+#include <dpkg-db.h>
+#include <tarfn.h>
+#include <myopt.h>
 
 #include "filesdb.h"
 #include "main.h"
@@ -57,18 +58,18 @@ void process_archive(const char *filename) {
   static int p1[2];
   static char cidirtmpnambuf[L_tmpnam+100];
   static char *cidirbuf=0, *reasmbuf=0;
-  static struct fileinlist *newconffiles;
+  static struct fileinlist *newconffiles, *newfileslist;
   static enum pkgstatus oldversionstatus;
   static struct varbuf infofnvb, fnvb, depprobwhy;
   static struct tarcontext tc;
   
   int c1, r, admindirlen, i, infodirlen, infodirbaseused, status;
   struct pkgiterator *it;
-  struct pkginfo *pkg, *conflictor, *otherpkg, *divpkg;
+  struct pkginfo *pkg, *otherpkg, *divpkg;
   char *cidir, *cidirrest, *p;
-  char pfilenamebuf[35], conffilenamebuf[MAXCONFFILENAME];
+  char *pfilenamebuf, conffilenamebuf[MAXCONFFILENAME];
   const char *pfilename, *newinfofilename;
-  struct fileinlist *newconff, **newconffileslastp, *newfileslist;
+  struct fileinlist *newconff, **newconffileslastp;
   struct fileinlist *cfile;
   struct reversefilelistiter rlistit;
   struct conffile *searchconff, **iconffileslastp, *newiconff;
@@ -82,18 +83,15 @@ void process_archive(const char *filename) {
   struct dirent *de;
   struct stat stab;
   struct packageinlist *deconpil, *deconpiltemp;
-  enum versiondisplayepochwhen needepochs;
   
   cleanup_pkg_failed= cleanup_conflictor_failed= 0;
   admindirlen= strlen(admindir);
 
-  pfilename= filename;
-  while (strlen(pfilename) > sizeof(pfilenamebuf)-5) {
+  for (pfilename= filename ; pfilename && strlen(pfilename) > 30 &&
+      strchr(pfilename,'/') != NULL ; pfilename++ )
     pfilename= strchr(pfilename,'/');
-    if (!pfilename) break;
-    pfilename++;
-  }
   if (pfilename && pfilename != filename) {
+    pfilenamebuf= (char *)nfmalloc(strlen(pfilename)+5);
     strcpy(pfilenamebuf,".../");
     strcat(pfilenamebuf,pfilename);
     pfilename= pfilenamebuf;
@@ -101,7 +99,7 @@ void process_archive(const char *filename) {
     pfilename= filename;
   }
 
-  if (stat(filename,&stab)) ohshite("cannot access archive");
+  if (stat(filename,&stab)) ohshite(_("cannot access archive"));
 
   if (!f_noact) {
     /* We can't `tentatively-reassemble' packages. */
@@ -111,21 +109,21 @@ void process_archive(const char *filename) {
       strcat(reasmbuf,"/" REASSEMBLETMP);
     }
     if (unlink(reasmbuf) && errno != ENOENT)
-      ohshite("error ensuring `%.250s' doesn't exist",reasmbuf);
+      ohshite(_("error ensuring `%.250s' doesn't exist"),reasmbuf);
     push_cleanup(cu_pathname,~0, 0,0, 1,(void*)reasmbuf);
     c1= m_fork();
     if (!c1) {
       execlp(SPLITTER, SPLITTER,"-Qao",reasmbuf,filename,(char*)0);
-      ohshite("failed to exec " SPLITTER " to see if it's part of a multiparter");
+      ohshite(_("failed to exec dpkg-split to see if it's part of a multiparter"));
     }
     while ((r= waitpid(c1,&status,0)) == -1 && errno == EINTR);
-    if (r != c1) { onerr_abort++; ohshite("wait for " SPLITTER " failed"); }
+    if (r != c1) { onerr_abort++; ohshite(_("wait for dpkg-split failed")); }
     switch (WIFEXITED(status) ? WEXITSTATUS(status) : -1) {
     case 0:
       /* It was a part - is it complete ? */
       if (!stat(reasmbuf,&stab)) { /* Yes. */
         filename= reasmbuf;
-        pfilename= "reassembled package file";
+        pfilename= _("reassembled package file");
         break;
       } else if (errno == ENOENT) { /* No.  That's it, we skip it. */
         return;
@@ -140,7 +138,7 @@ void process_archive(const char *filename) {
   
   if (f_noact) {
     cidir= cidirtmpnambuf;
-    if (!tmpnam(cidir)) ohshite("unable to get unique filename for control info");
+    if (!tmpnam(cidir)) ohshite(_("unable to get unique filename for control info"));
     strcat(cidir,"/");
   } else {
     /* We want it to be on the same filesystem so that we can
@@ -162,21 +160,23 @@ void process_archive(const char *filename) {
   if (!c1) {
     cidirrest[-1]= 0;
     execlp(BACKEND, BACKEND,"--control",filename,cidir,(char*)0);
-    ohshite("failed to exec " BACKEND " to extract control information");
+    ohshite(_("failed to exec dpkg-deb to extract control information"));
   }
   waitsubproc(c1,BACKEND " --control",0);
   strcpy(cidirrest,CONTROLFILE);
 
-  parsedb(cidir, pdb_recordavailable|pdb_rejectstatus|pdb_weakclassification,
+  parsedb(cidir, pdb_recordavailable|pdb_rejectstatus|pdb_ignorefiles|pdb_weakclassification,
           &pkg,0,0);
-  pkg->files= nfmalloc(sizeof(struct filedetails));
-  pkg->files->next= 0;
-  pkg->files->name= pkg->files->msdosname= pkg->files->md5sum= 0;
-  pkg->files->size= nfmalloc(30);
+  if (!pkg->files) {
+    pkg->files= nfmalloc(sizeof(struct filedetails));
+    pkg->files->next= 0;
+    pkg->files->name= pkg->files->msdosname= pkg->files->md5sum= 0;
+    pkg->files->size= nfmalloc(30);
+  }
   sprintf(pkg->files->size,"%lu",(unsigned long)stab.st_size);
 
   if (cipaction->arg == act_avail) {
-    printf("Recorded info about %s from %s.\n",pkg->name,pfilename);
+    printf(_("Recorded info about %s from %s.\n"),pkg->name,pfilename);
     pop_cleanup(ehflag_normaltidy);
     return;
   }
@@ -185,7 +185,7 @@ void process_archive(const char *filename) {
       strcmp(pkg->available.architecture,"all") &&
       strcmp(pkg->available.architecture,architecture))
     forcibleerr(fc_architecture,
-                "package architecture (%s) does not match system (%s)",
+                _("package architecture (%s) does not match system (%s)"),
                 pkg->available.architecture,architecture);
     
   if (!pkg->installed.valid) blankpackageperfile(&pkg->installed);
@@ -200,52 +200,20 @@ void process_archive(const char *filename) {
   deconfigure= 0;
   clear_istobes();
 
-  if (pkg->want != want_install) {
-    if (f_alsoselect) {
-      printf("Selecting previously deselected package %s.\n",pkg->name);
-      pkg->want= want_install;
-    } else {
-      printf("Skipping deselected package %s.\n",pkg->name);
-      return;
-    }
-  }
-
-  if (pkg->status == stat_installed) {
-    r= versioncompare(&pkg->available.version,&pkg->installed.version);
-    if (r < 0) {
-      needepochs= epochsdiffer(&pkg->available.version,&pkg->installed.version) ?
-        vdew_always : vdew_never;
-      if (fc_downgrade) {
-        fprintf(stderr, DPKG " - warning: downgrading %.250s from %.250s to %.250s.\n",
-                pkg->name,
-                versiondescribe(&pkg->installed.version,needepochs),
-                versiondescribe(&pkg->available.version,needepochs));
-      } else {
-        fprintf(stderr, "Will not downgrade"
-                " %.250s from version %.250s to %.250s, skipping.\n",
-                pkg->name,
-                versiondescribe(&pkg->installed.version,needepochs),
-                versiondescribe(&pkg->available.version,needepochs));
-        pop_cleanup(ehflag_normaltidy);
-        return;
-      }
-    } else if (r == 0 && f_skipsame && /* same version fully installed ? */
-               pkg->status == stat_installed && !(pkg->eflag &= eflagf_reinstreq)) {
-      fprintf(stderr, "Version %.250s of %.250s already installed, skipping.\n",
-              versiondescribe(&pkg->installed.version,vdew_never),
-              pkg->name);
+  if (!wanttoinstall(pkg,&pkg->available.version,1)) {
       pop_cleanup(ehflag_normaltidy);
       return;
-    }
   }
 
+  /* Check if anything is installed that we conflict with, or not installed
+   * that we need */
   pkg->clientdata->istobe= itb_installnew;
-  conflictor= 0;
+
   for (dsearch= pkg->available.depends; dsearch; dsearch= dsearch->next) {
     switch (dsearch->type) {
     case dep_conflicts:
       /* Look for things we conflict with. */
-      check_conflict(dsearch, pkg, pfilename, &conflictor);
+      check_conflict(dsearch, pkg, pfilename);
       break;
     case dep_provides:
       /* Look for things that conflict with what we provide. */
@@ -254,40 +222,44 @@ void process_archive(const char *filename) {
              psearch;
              psearch= psearch->nextrev) {
           if (psearch->up->type != dep_conflicts) continue;
-          check_conflict(psearch->up, pkg, pfilename, &conflictor);
+          check_conflict(psearch->up, pkg, pfilename);
         }
       }
       break;
-    case dep_suggests: case dep_recommends: case dep_depends: case dep_replaces:
+    case dep_suggests:
+    case dep_recommends:
+    case dep_depends:
+    case dep_replaces:
+    case dep_enhances:
       /* Ignore these here. */
       break;
     case dep_predepends:
       if (!depisok(dsearch,&depprobwhy,0,1)) {
         varbufaddc(&depprobwhy,0);
-        fprintf(stderr, DPKG ": regarding %s containing %s, pre-dependency problem:\n%s",
+        fprintf(stderr, _("dpkg: regarding %s containing %s, pre-dependency problem:\n%s"),
                 pfilename, pkg->name, depprobwhy.buf);
         if (!force_depends(dsearch->list))
-          ohshit("pre-dependency problem - not installing %.250s",pkg->name);
-        fprintf(stderr, DPKG ": warning - ignoring pre-dependency problem !\n");
+          ohshit(_("pre-dependency problem - not installing %.250s"),pkg->name);
+        fprintf(stderr, _("dpkg: warning - ignoring pre-dependency problem !\n"));
       }
     }
   }
   /* Look for things that conflict with us. */
   for (psearch= pkg->installed.depended; psearch; psearch= psearch->nextrev) {
     if (psearch->up->type != dep_conflicts) continue;
-    check_conflict(psearch->up, pkg, pfilename, &conflictor);
+    check_conflict(psearch->up, pkg, pfilename);
   }
   
   ensure_allinstfiles_available();
   filesdbinit();
   
   if (pkg->status != stat_notinstalled && pkg->status != stat_configfiles)
-    printf("Preparing to replace %s %s (using %s) ...\n",
+    printf(_("Preparing to replace %s %s (using %s) ...\n"),
            pkg->name,
            versiondescribe(&pkg->installed.version,vdew_nonambig),
            pfilename);
   else
-    printf("Unpacking %s (from %s) ...\n",pkg->name,pfilename);
+    printf(_("Unpacking %s (from %s) ...\n"),pkg->name,pfilename);
 
   if (f_noact) {
     pop_cleanup(ehflag_normaltidy);
@@ -307,14 +279,14 @@ void process_archive(const char *filename) {
       p= conffilenamebuf + strlen(conffilenamebuf);
       assert(p != conffilenamebuf);
       if (p[-1] != '\n')
-        ohshit("name of conffile (starting `%.250s') is too long (>%d characters)",
+        ohshit(_("name of conffile (starting `%.250s') is too long (>%d characters)"),
                conffilenamebuf, MAXCONFFILENAME);
       while (p > conffilenamebuf && isspace(p[-1])) --p;
       if (p == conffilenamebuf) continue;
       *p= 0;
       newconff= m_malloc(sizeof(struct fileinlist));
       newconff->next= 0;
-      newconff->namenode= findnamenode(conffilenamebuf);
+      newconff->namenode= findnamenode(conffilenamebuf, 0);
       *newconffileslastp= newconff;
       newconffileslastp= &newconff->next;
       newconff->namenode->oldhash= NEWCONFFILEFLAG;
@@ -361,24 +333,26 @@ void process_archive(const char *filename) {
       }
       newconff->namenode->flags |= fnnf_new_conff;
     }
-    if (ferror(conff)) ohshite("read error in %.250s",cidir);
+    if (ferror(conff)) ohshite(_("read error in %.250s"),cidir);
     pop_cleanup(ehflag_normaltidy); /* conff= fopen() */
-    if (fclose(conff)) ohshite("error closing %.250s",cidir);
+    if (fclose(conff)) ohshite(_("error closing %.250s"),cidir);
   } else {
-    if (errno != ENOENT) ohshite("error trying to open %.250s",cidir);
+    if (errno != ENOENT) ohshite(_("error trying to open %.250s"),cidir);
   }
 
   /* All the old conffiles are marked with a flag, so that we don't delete
    * them if they seem to disappear completely.
    */
   oldconffsetflags(pkg->installed.conffiles);
-  if (conflictor) oldconffsetflags(conflictor->installed.conffiles);
+  for (i = 0 ; i < cflict_index ; i++) {
+    oldconffsetflags(conflictor[i]->installed.conffiles);
+  }
   
   oldversionstatus= pkg->status;
 
   assert(oldversionstatus <= stat_configfiles);
-  debug(dbg_general,"process_archive oldversionstatus=%s conflictor=%s",
-        statusstrings[oldversionstatus], conflictor ? conflictor->name : "<none>");
+  debug(dbg_general,"process_archive oldversionstatus=%s",
+        statusstrings[oldversionstatus]);
   
   if (oldversionstatus == stat_halfconfigured || oldversionstatus == stat_installed) {
     pkg->eflag |= eflagf_reinstreq;
@@ -392,13 +366,12 @@ void process_archive(const char *filename) {
     modstatdb_note(pkg);
   }
 
-  if (conflictor &&
-      (conflictor->status == stat_halfconfigured ||
-       conflictor->status == stat_installed)) {
-
+  for (i = 0 ; i < cflict_index ; i++) {
+    if (!(conflictor[i]->status == stat_halfconfigured ||
+	  conflictor[i]->status == stat_installed)) continue;
     for (deconpil= deconfigure; deconpil; deconpil= deconpil->next) {
-      printf("De-configuring %s, so that we can remove %s ...\n",
-             deconpil->pkg->name, conflictor->name);
+      printf(_("De-configuring %s, so that we can remove %s ...\n"),
+             deconpil->pkg->name, conflictor[i]->name);
       deconpil->pkg->status= stat_halfconfigured;
       modstatdb_note(deconpil->pkg);
       /* This means that we *either* go and run postinst abort-deconfigure,
@@ -407,27 +380,28 @@ void process_archive(const char *filename) {
        */
       push_cleanup(cu_prermdeconfigure,~ehflag_normaltidy,
                    ok_prermdeconfigure,ehflag_normaltidy,
-                   3,(void*)deconpil->pkg,(void*)conflictor,(void*)pkg);
+                   3,(void*)deconpil->pkg,
+		   (void*)conflictor[i],(void*)pkg);
       maintainer_script_installed(deconpil->pkg, PRERMFILE, "pre-removal",
                                   "deconfigure", "in-favour", pkg->name,
                                   versiondescribe(&pkg->available.version,
                                                   vdew_nonambig),
-                                  "removing", conflictor->name,
-                                  versiondescribe(&conflictor->installed.version,
+                                  "removing", conflictor[i]->name,
+                                  versiondescribe(&conflictor[i]->installed.version,
                                                   vdew_nonambig),
                                   (char*)0);
     }
-    conflictor->status= stat_halfconfigured;
-    modstatdb_note(conflictor);
+    conflictor[i]->status= stat_halfconfigured;
+    modstatdb_note(conflictor[i]);
     push_cleanup(cu_prerminfavour,~ehflag_normaltidy, 0,0,
-                 2,(void*)conflictor,(void*)pkg);
-    maintainer_script_installed(conflictor, PRERMFILE, "pre-removal",
+                 2,(void*)conflictor[i],(void*)pkg);
+    maintainer_script_installed(conflictor[i], PRERMFILE, "pre-removal",
                                 "remove", "in-favour", pkg->name,
                                 versiondescribe(&pkg->available.version,
                                                 vdew_nonambig),
                                 (char*)0);
-    conflictor->status= stat_halfinstalled;
-    modstatdb_note(conflictor);
+    conflictor[i]->status= stat_halfinstalled;
+    modstatdb_note(conflictor[i]);
   }
 
   pkg->eflag |= eflagf_reinstreq;
@@ -454,7 +428,7 @@ void process_archive(const char *filename) {
                           "upgrade", versiondescribe(&pkg->installed.version,
                                                      vdew_nonambig),
                           (char*)0);
-    printf("Unpacking replacement %.250s ...\n",pkg->name);
+    printf(_("Unpacking replacement %.250s ...\n"),pkg->name);
   }
   
   /*
@@ -506,6 +480,9 @@ void process_archive(const char *filename) {
    *  - The listed thing does not exist.  We ignore it.
    *  - The listed thing is a directory or a symlink to a directory.
    *    We delete it only if it isn't listed in any other package.
+   *  - The listed thing is not a directory, but was part of the package
+   *    that was upgraded, we check to make sure the files aren't the
+   *    same ones from the old package by checking dev/inode
    *  - The listed thing is not a directory or a symlink to one (ie,
    *    it's a plain file, device, pipe, &c, or a symlink to one, or a
    *    dangling symlink).  We delete it.
@@ -530,7 +507,7 @@ void process_archive(const char *filename) {
   if (!c1) {
     m_dup2(p1[1],1); close(p1[0]); close(p1[1]);
     execlp(BACKEND, BACKEND, "--fsys-tarfile", filename, (char*)0);
-    ohshite("unable to exec " BACKEND " to get filesystem archive");
+    ohshite(_("unable to exec dpkg-deb to get filesystem archive"));
   }
   close(p1[1]);
 
@@ -538,18 +515,18 @@ void process_archive(const char *filename) {
   push_cleanup(cu_fileslist,~0, 0,0, 1,(void*)&newfileslist);
   tc.pkg= pkg;
   tc.backendpipe= fdopen(p1[0],"r");
-  if (!tc.backendpipe) ohshite("unable to fdopen " BACKEND " extract pipe");
+  if (!tc.backendpipe) ohshite(_("unable to fdopen dpkg-deb extract pipe"));
   push_cleanup(cu_backendpipe,~ehflag_bombout, 0,0, 1,(void*)&tc.backendpipe);
 
   r= TarExtractor((void*)&tc, &tf);
   if (r) {
     if (errno) {
-      ohshite("error reading " BACKEND " tar output");
+      ohshite(_("error reading dpkg-deb tar output"));
     } else if (feof(tc.backendpipe)) {
       waitsubproc(c1,BACKEND " --fsys-tarfile (EOF)",1);
-      ohshit("unexpected EOF in filesystem tarfile - corrupted package archive");
+      ohshite(_("unexpected EOF in filesystem tarfile - corrupted package archive"));
     } else {
-      ohshit("corrupted filesystem tarfile - corrupted package archive");
+      ohshite(_("corrupted filesystem tarfile - corrupted package archive"));
     }
   }
   tmpf= tc.backendpipe;
@@ -602,11 +579,50 @@ void process_archive(const char *filename) {
     if (!rmdir(fnamevb.buf)) continue;
     if (errno == ENOENT || errno == ELOOP) continue;
     if (errno == ENOTDIR) {
+      /* Ok, it's an old file, but is it really not in the new package?
+       * We need to check to make sure, so we stat the file, then compare
+       * it to the new list. If we find a dev/inode match, we assume they
+       * are the same file, and leave it alone. NOTE: we don't check in
+       * other packages for sanity reasons (we don't want to stat _all_
+       * the files on the system).
+       *
+       * We run down the list of _new_ files in this package. This keeps
+       * the process a little leaner. We are only worried about new ones
+       * since ones that stayed the same don't really apply here.
+       */
+      struct stat oldfs;
+      int donotrm = 0;
+      /* If we can't stat the old or new file, or it's a directory,
+       * we leave it up to the normal code
+       */
+      debug(dbg_eachfile, "process_archive: checking %s for same files on "
+	  "upgrade/downgrade", fnamevb.buf);
+      if (!lstat(fnamevb.buf, &oldfs) && !S_ISDIR(oldfs.st_mode)) {
+	for (cfile = newfileslist; cfile; cfile = cfile->next) {
+	  if (!cfile->namenode->filestat) {
+	    cfile->namenode->filestat = (struct stat *) nfmalloc(sizeof(struct stat));
+	    if (lstat(cfile->namenode->name, cfile->namenode->filestat)) {
+	      cfile->namenode->filestat= 0;
+	      continue;
+	    }
+	  }
+	  if (S_ISDIR(cfile->namenode->filestat->st_mode))
+	    continue;
+          if (oldfs.st_dev == cfile->namenode->filestat->st_dev &&
+              oldfs.st_ino == cfile->namenode->filestat->st_ino) {
+	    donotrm = 1;
+	    debug(dbg_eachfile, "process_archive: not removing %s, since it matches %s",
+		fnamevb.buf, cfile->namenode->name);
+	  }
+	}
+      } else
+	debug(dbg_eachfile, "process_archive: could not stat %s, skipping", fnamevb.buf);
+      if (donotrm) continue;
       if (!unlink(fnamevb.buf)) continue;
       if (errno == ENOTDIR) continue;
     }
     fprintf(stderr,
-            DPKG ": warning - unable to delete old file `%.250s': %s\n",
+            _("dpkg: warning - unable to delete old file `%.250s': %s\n"),
             namenode->name, strerror(errno));
   }
 
@@ -628,7 +644,7 @@ void process_archive(const char *filename) {
   infodirlen= infofnvb.used;
   varbufaddc(&infofnvb,0);
   dsd= opendir(infofnvb.buf);
-  if (!dsd) ohshite("cannot read info directory");
+  if (!dsd) ohshite(_("cannot read info directory"));
   push_cleanup(cu_closedir,~0, 0,0, 1,(void*)dsd);
   while ((de= readdir(dsd)) != 0) {
     debug(dbg_veryverbose, "process_archive info file `%s'", de->d_name);
@@ -641,7 +657,7 @@ void process_archive(const char *filename) {
     p++; /* skip past the full stop */
     if (!strcmp(p,LISTFILE)) continue; /* We do the list separately */
     if (strlen(p) > MAXCONTROLFILENAME)
-      ohshit("old version of package has overly-long info file name starting `%.250s'",
+      ohshit(_("old version of package has overly-long info file name starting `%.250s'"),
              de->d_name);
     infofnvb.used= infodirlen;
     varbufaddstr(&infofnvb,de->d_name);
@@ -653,17 +669,17 @@ void process_archive(const char *filename) {
     } else if (errno == ENOENT) {
       /* Right, no new version. */
       if (unlink(infofnvb.buf))
-        ohshite("unable to remove obsolete info file `%.250s'",infofnvb.buf);
+        ohshite(_("unable to remove obsolete info file `%.250s'"),infofnvb.buf);
       debug(dbg_scripts, "process_archive info unlinked %s",infofnvb.buf);
     } else {
-      ohshite("unable to install (supposed) new info file `%.250s'",cidir);
+      ohshite(_("unable to install (supposed) new info file `%.250s'"),cidir);
     }
   }
   pop_cleanup(ehflag_normaltidy); /* closedir */
   
   *cidirrest= 0; /* the directory itself */
   dsd= opendir(cidir);
-  if (!dsd) ohshite("unable to open temp control directory");
+  if (!dsd) ohshite(_("unable to open temp control directory"));
   push_cleanup(cu_closedir,~0, 0,0, 1,(void*)dsd);
   while ((de= readdir(dsd))) {
     if (strchr(de->d_name,'.')) {
@@ -672,27 +688,27 @@ void process_archive(const char *filename) {
       continue;
     }
     if (strlen(de->d_name) > MAXCONTROLFILENAME)
-      ohshit("package contains overly-long control info file name (starting `%.50s')",
+      ohshit(_("package contains overly-long control info file name (starting `%.50s')"),
              de->d_name);
     strcpy(cidirrest,de->d_name);
     /* First we check it's not a directory. */
     if (!rmdir(cidir))
-      ohshit("package control info contained directory `%.250s'",cidir);
+      ohshit(_("package control info contained directory `%.250s'"),cidir);
     else if (errno != ENOTDIR)
-      ohshite("package control info rmdir of `%.250s' didn't say not a dir",de->d_name);
+      ohshite(_("package control info rmdir of `%.250s' didn't say not a dir"),de->d_name);
     if (!strcmp(de->d_name,CONTROLFILE)) {
       debug(dbg_scripts,"process_archive tmp.ci script/file `%s' is control",cidir);
       continue; /* ignore the control file */
     }
     if (!strcmp(de->d_name,LISTFILE)) {
-      fprintf(stderr, DPKG ": warning - package %s"
-              " contained list as info file", pkg->name);
+      fprintf(stderr, _("dpkg: warning - package %s"
+              " contained list as info file"), pkg->name);
       continue;
     }
     /* Right, install it */
     newinfofilename= pkgadminfile(pkg,de->d_name);
     if (rename(cidir,newinfofilename))
-      ohshite("unable to install new info file `%.250s' as `%.250s'",
+      ohshite(_("unable to install new info file `%.250s' as `%.250s'"),
               cidir,newinfofilename);
     debug(dbg_scripts,"process_archive tmp.ci script/file `%s' installed as `%s'",
           cidir, newinfofilename);
@@ -787,9 +803,9 @@ void process_archive(const char *filename) {
   while ((otherpkg= iterpkgnext(it)) != 0) {
     ensure_package_clientdata(otherpkg);
     if (otherpkg == pkg ||
-        otherpkg == conflictor ||
         otherpkg->status == stat_notinstalled ||
         otherpkg->status == stat_configfiles ||
+	otherpkg->clientdata->istobe == itb_remove ||
         !otherpkg->clientdata->files) continue;
     debug(dbg_veryverbose, "process_archive checking disappearance %s",otherpkg->name);
     assert(otherpkg->clientdata->istobe == itb_normal ||
@@ -843,7 +859,7 @@ void process_archive(const char *filename) {
     otherpkg->clientdata->istobe= itb_normal;
     if (pdep) continue;
 
-    printf("(Noting disappearance of %s, which has been completely replaced.)\n",
+    printf(_("(Noting disappearance of %s, which has been completely replaced.)\n"),
            otherpkg->name);
     debug(dbg_general, "process_archive disappearing %s",otherpkg->name);
     /* No, we're disappearing it.  This is the wrong time to go and
@@ -863,7 +879,7 @@ void process_archive(const char *filename) {
     varbufaddstr(&fnvb,"/" INFODIR);
     infodirbaseused= fnvb.used;
     varbufaddc(&fnvb,0);
-    dsd= opendir(fnvb.buf); if (!dsd) ohshite("cannot read info directory");
+    dsd= opendir(fnvb.buf); if (!dsd) ohshite(_("cannot read info directory"));
     push_cleanup(cu_closedir,~0, 0,0, 1,(void*)dsd);
 
     debug(dbg_general, "process_archive disappear cleaning info directory");
@@ -879,7 +895,7 @@ void process_archive(const char *filename) {
       varbufaddstr(&fnvb,de->d_name);
       varbufaddc(&fnvb,0);
       if (unlink(fnvb.buf))
-        ohshite("unable to delete disappearing control info file `%.250s'",fnvb.buf);
+        ohshite(_("unable to delete disappearing control info file `%.250s'"),fnvb.buf);
       debug(dbg_scripts, "process_archive info unlinked %s",fnvb.buf);
     }
     pop_cleanup(ehflag_normaltidy); /* closedir */
@@ -992,10 +1008,10 @@ void process_archive(const char *filename) {
    * as we have not yet updated the filename->packages mappings; however,
    * the package->filenames mapping is 
    */
-  if (conflictor) {
+  for (i = 0 ; i < cflict_index ; i++) {
     /* We need to have the most up-to-date info about which files are what ... */
     ensure_allinstfiles_available();
-    removal_bulk(conflictor);
+    removal_bulk(conflictor[i]);
   }
 
   if (cipaction->arg == act_install) add_to_queue(pkg);
