@@ -17,7 +17,7 @@
 ;; GNU General Public License for more details.
 ;;
 ;; You should have received a copy of the GNU General Public License
-;; along with your Debian installation, in /usr/doc/copyright/GPL.
+;; along with your Debian installation, in /usr/share/common-licenses/GPL.
 ;; If not, write to the Free Software Foundation, 675 Mass Ave,
 ;; Cambridge, MA 02139, USA.
 
@@ -218,7 +218,20 @@ Runs `debian-changelog-mode-hook' if it exists.
 
 Key bindings:
 
-\\{debian-changelog-mode-map}"
+\\{debian-changelog-mode-map}
+Hint:  If you want to use your debian.org email address for debian/changelog
+entries without using it for the rest of your email, put
+
+    (add-hook 'debian-changelog-mode-hook 'my-debian-changelog-mode-hook)
+    (defun my-debian-changelog-mode-hook ()
+      (make-local-variable 'add-log-mailing-address)
+      (setq add-log-mailing-address \"myname@debian.org\"))
+
+in your ~/.emacs file.  This is better than including a local setting
+for add-log-mailing-address in the changelog itself both because you
+only have to set it up once and because it works in the presense of
+NMUs."
+
   (interactive)
   (kill-all-local-variables)
   (text-mode)
