@@ -49,7 +49,8 @@ Options:  -b                     binary-only build - no source files
 
 $i=100;grep($fieldimps{$_}=$i--,
           qw(Format Date Source Binary Architecture Version
-             Distribution Urgency Maintainer Description Changes Files));
+             Distribution Urgency Maintainer Description Closes Changes
+	     Files));
 
 while (@ARGV) {
     $_=shift(@ARGV);
@@ -113,6 +114,12 @@ while(<FL>) {
         $f2sec{$1}= $5;
         $f2pri{$1}= $6;
         push(@fileslistfiles,$1);
+    } elsif (m/^([-+.0-9a-z]+_[^_]+_([-\w]+)\.[a-z0-9.]+) (\S+) (\S+)$/) {
+	# A non-deb package
+	$f2sec{$1}= $3;
+	$f2pri{$1}= $4;
+	push(@archvalues,$2) unless !$2 || $archadded{$2}++;
+	push(@fileslistfiles,$1);
     } elsif (m/^([-+.,_0-9a-zA-Z]+) (\S+) (\S+)$/) {
 	defined($f2sec{$1}) &&
             &warn("duplicate files list entry for file $1 (line $.)");
@@ -162,7 +169,7 @@ for $_ (keys %fi) {
 		}
 		push(@archvalues,$v) unless !$v || $archadded{$v}++;
 	    } elsif (m/^(Package|Essential|Pre-Depends|Depends|Provides)$/ ||
-		     m/^(Recommends|Suggests|Optional|Conflicts|Replaces)$/ ||
+		     m/^(Recommends|Suggests|Enhances|Optional|Conflicts|Replaces)$/ ||
 		     m/^X[CS]+-/i) {
 	    } else {
 		&unknown("package's section of control info file");
@@ -172,7 +179,7 @@ for $_ (keys %fi) {
 #print STDERR "L key >$_< value >$v<\n";
         if (m/^Source$/) {
             &setsourcepackage;
-        } elsif (m/^(Version|Maintainer|Changes|Urgency|Distribution|Date)$/) {
+        } elsif (m/^(Version|Maintainer|Changes|Urgency|Distribution|Date|Closes)$/) {
             $f{$_}= $v;
         } elsif (s/^X[BS]*C[BS]*-//i) {
             $f{$_}= $v;
@@ -237,7 +244,7 @@ if (!$binaryonly) {
     }
     for $f (@sourcefiles) { $f2sec{$f}= $sec; $f2pri{$f}= $pri; }
     
-    if (($sourcestyle =~ m/i/ && $version !~ m/-[01]$/ ||
+    if (($sourcestyle =~ m/i/ && $version !~ m/-(0|1|0\.1)$/ ||
          $sourcestyle =~ m/d/) &&
         grep(m/\.diff\.gz$/,@sourcefiles)) {
         $origsrcmsg= "not including original source code in upload";
