@@ -8,6 +8,8 @@ my %dirincluded;
 my %notfileobject;
 my $fn;
 
+$diff_ignore_default_regexp = '^.*~$|DEAD_JOE|(?:/CVS|/RCS|/.deps)(?:$|/.*$)';
+
 $sourcestyle = 'X';
 
 use POSIX;
@@ -18,6 +20,9 @@ use strict 'refs';
 push (@INC, $dpkglibdir);
 require 'controllib.pl';
 
+# Make sure patch doesn't get any funny ideas
+delete $ENV{'POSIXLY_CORRECT'};
+
 sub usageversion {
     print STDERR
 "Debian GNU/Linux dpkg-source $version.  Copyright (C) 1996
@@ -35,6 +40,8 @@ Build options:   -c<controlfile>     get control info from this file
                  -D<field>=<value>   override or add a .dsc field and value
                  -U<field>           remove a field
                  -sa                 auto select orig source (-sA is default)
+                 -i[<regexp>]        filter out files to ignore diffs of.
+                                     Defaults to: '$diff_ignore_default_regexp'
                  -sk                 use packed orig source (unpack & keep)
                  -sp                 use packed orig source (unpack & remove)
                  -su                 use unpacked orig source (pack & keep)
@@ -51,7 +58,8 @@ General options: -h                  print this message
 
 $i = 100;
 grep ($fieldimps {$_} = $i--,
-      qw (Source Version Binary Maintainer Architecture Standards-Version));
+      qw (Source Version Binary Maintainer Architecture Standards-Version
+          Build-Depends Build-Depends-Indep Build-Conflicts Build-Conflicts-Indep));
 
 while (@ARGV && $ARGV[0] =~ m/^-/) {
     $_=shift(@ARGV);
@@ -71,6 +79,8 @@ while (@ARGV && $ARGV[0] =~ m/^-/) {
         $override{$1}= $';
     } elsif (m/^-U([^\=:]+)$/) {
         $remove{$1}= 1;
+    } elsif (m/^-i(.*)$/) {
+        $diff_ignore_regexp = $1 ? $1 : $diff_ignore_default_regexp;
     } elsif (m/^-V(\w[-:0-9A-Za-z]*)[=:]/) {
         $substvar{$1}= $';
     } elsif (m/^-T/) {
@@ -112,6 +122,7 @@ if ($opmode eq 'build') {
 #print STDERR "G key >$_< value >$v<\n";
             if (m/^Source$/) { &setsourcepackage; }
             elsif (m/^Standards-Version$|^Maintainer$/) { $f{$_}= $v; }
+	    elsif (m/^Build-(Depends|Conflicts)(-Indep)?$/i) { $f{$_}= $v; }
             elsif (s/^X[BC]*S[BC]*-//i) { $f{$_}= $v; }
             elsif (m/^(Section|Priority|Files)$/ || m/^X[BC]+-/i) { }
             else { &unknown('general section of control info file'); }
@@ -159,7 +170,7 @@ if ($opmode eq 'build') {
                 $f{$_}= $v;
             } elsif (s/^X[BS]*C[BS]*-//i) {
                 $f{$_}= $v;
-            } elsif (m/^(Maintainer|Changes|Urgency|Distribution|Date)$/ ||
+            } elsif (m/^(Maintainer|Changes|Urgency|Distribution|Date|Closes)$/ ||
                      m/^X[BS]+-/i) {
             } else {
                 &unknown("parsed version of changelog");
@@ -338,7 +349,9 @@ if ($opmode eq 'build') {
 
       file:
         while (defined($fn= <FIND>)) {
-            $fn =~ s/\0$//; $fn =~ s,^\./,,;
+            $fn =~ s/\0$//;
+            next file if $fn =~ m/$diff_ignore_regexp/o;
+            $fn =~ s,^\./,,;
             lstat("$dir/$fn") || &syserr("cannot stat file $dir/$fn");
             if (-l _) {
                 $type{$fn}= 'symlink';
@@ -361,6 +374,7 @@ if ($opmode eq 'build') {
                 }
                 defined($c3= open(DIFFGEN,"-|")) || &syserr("fork for diff");
                 if (!$c3) {
+		    $ENV{'LANG'}= 'C';
                     exec('diff','-u',
                          '-L',"$basedirname.orig/$fn",
                          '-L',"$basedirname/$fn",
@@ -414,7 +428,9 @@ if ($opmode eq 'build') {
         }
         $/= "\0";
         while (defined($fn= <FIND>)) {
-            $fn =~ s/\0$//; $fn =~ s,^\./,,;
+            $fn =~ s/\0$//;
+            next if $fn =~ m/$diff_ignore_regexp/o;
+            $fn =~ s,^\./,,;
             next if defined($type{$fn});
             lstat("$origdir/$fn") || &syserr("cannot check orig file $origdir/$fn");
             if (-f _) {
@@ -453,7 +469,7 @@ if ($opmode eq 'build') {
 
     $sourcestyle =~ y/X/p/;
     $sourcestyle =~ m/[pun]/ ||
-        &usageerr("source handling style -s$sourcestyle not allowed with -b");
+        &usageerr("source handling style -s$sourcestyle not allowed with -x");
 
     @ARGV==1 || &usageerr("-x needs exactly one argument, the .dsc");
     $dsc= shift(@ARGV);
@@ -509,9 +525,11 @@ if ($opmode eq 'build') {
             
         &forkgzipread("$dscdir/$difffile");
         $/="\n";
-        while (<GZIP>) {
-            s/\n$// || &error("diff is missing trailing newline");
-            if (/^--- /) {
+	$_ = <GZIP>;
+        while ($_ || !eof(GZIP)) {
+	    # read file header (---/+++ pair)
+	    s/\n$// or &error("diff is missing trailing newline");
+	    /^--- / or &error("expected ^--- in line $. of diff");
                 $fn= $';
                 substr($fn,0,length($expectprefix)+1) eq "$expectprefix/" ||
                     &error("diff patches file ($fn) not in expected subdirectory");
@@ -528,11 +546,28 @@ if ($opmode eq 'build') {
                 $_ eq '+++ '.$newdirectory.substr($fn,length($expectprefix)) ||
                     &error("line after --- for file $fn isn't as expected");
                 $filepatched{$fn}++ && &error("diff patches file $fn twice");
-            } elsif (/^\\ No newline at end of file$/) {
-            } elsif (/^[-+ \@]/) {
-	    } else {
-                &error ("diff contains unknown line \`$_'");
-            }
+	    # read hunks
+	    my $hunk = 0;
+	    while (($_ = <GZIP>) && !/^--- /) {
+		# read hunk header (@@)
+		s/\n$// or &error("diff is missing trailing newline");
+		next if /^\\ No newline/;
+		/^@@ -\d+(,(\d+))? \+\d+(,(\d+))? @\@$/ or
+		    &error("Expected ^@@ in line $. of diff");
+		my ($olines, $nlines) = ($1 ? $2 : 1, $3 ? $4 : 1);
+		++$hunk;
+		# read hunk
+		while ($olines || $nlines) {
+		    $_ = <GZIP> or &error("unexpected end of diff");
+		    s/\n$// or &error("diff is missing trailing newline");
+		    next if /^\\ No newline/;
+		    if (/^ /) { --$olines; --$nlines; }
+		    elsif (/^-/) { --$olines; }
+		    elsif (/^\+/) { --$nlines; }
+		    else { &error("expected [ +-] at start of line $. of diff"); }
+		}
+	    }
+	    $hunk or &error("expected ^\@\@ at line $. of diff");
         }
         close(GZIP);
         
@@ -547,6 +582,7 @@ if ($opmode eq 'build') {
 
     mkdir("$expectprefix.tmp-nest",0755)
 	|| &syserr("unable to create \`$expectprefix.tmp-nest'");
+    system "chmod", "g-s", "$expectprefix.tmp-nest";
     extracttar("$dscdir/$tarfile","$expectprefix.tmp-nest","$expectprefix");
     rename("$expectprefix.tmp-nest/$expectprefix","$expectprefix")
 	|| &syserr("unable to rename \`$expectprefix.tmp-nest/$expectprefix' "
@@ -577,6 +613,7 @@ if ($opmode eq 'build') {
         if ($sourcestyle =~ m/u/) {
 	    mkdir("$expectprefix.tmp-nest",0755)
 		|| &syserr("unable to create \`$expectprefix.tmp-nest'");
+	    system "chmod", "g-s", "$expectprefix.tmp-nest";
 	    extracttar("$dscdir/$tarfile","$expectprefix.tmp-nest",
 		       "$expectprefix");
 	    rename("$expectprefix.tmp-nest/$expectprefix","$expectprefix")
@@ -609,8 +646,9 @@ if ($opmode eq 'build') {
         if (!$c2) {
             open(STDIN,"<&GZIP") || &syserr("reopen gzip for patch");
             chdir($newdirectory) || &syserr("chdir to $newdirectory for patch");
+	    $ENV{'LANG'}= 'C';
             exec('patch','-s','-t','-F','0','-N','-p1','-u',
-                 '-V','never','-b','-z','.dpkg-orig');
+                 '-V','never','-g0','-b','-z','.dpkg-orig');
             &syserr("exec patch");
         }
         close(GZIP);
@@ -895,11 +933,19 @@ sub extracttar {
     opendir(D,"$dirchdir") || &syserr("Unable to open dir $dirchdir");
     @dirchdirfiles = grep($_ ne "." && $_ ne "..",readdir(D));
     closedir(D) || &syserr("Unable to close dir $dirchdir");
-    (@dirchdirfiles==1 && -d "$dirchdir/$dirchdirfiles[0]") ||
-	&error("$tarfileread extracted into >1 directory");
-    rename("$dirchdir/$dirchdirfiles[0]", "$dirchdir/$newtopdir") ||
-	&syserr("Unable to rename $dirchdir/$dirchdirfiles[0] to ".
-		"$dirchdir/$newtopdir");
+    if (@dirchdirfiles==1 && -d "$dirchdir/$dirchdirfiles[0]") {
+	rename("$dirchdir/$dirchdirfiles[0]", "$dirchdir/$newtopdir") ||
+	    &syserr("Unable to rename $dirchdir/$dirchdirfiles[0] to ".
+	    "$dirchdir/$newtopdir");
+    } else {
+	mkdir("$dirchdir/$newtopdir", 0777) ||
+	    &syserr("Unable to mkdir $dirchdir/$newtopdir");
+	for (@dirchdirfiles) {
+	    rename("$dirchdir/$_", "$dirchdir/$newtopdir/$_") ||
+		&syserr("Unable to rename $dirchdir/$_ to ".
+		"$dirchdir/$newtopdir/$_");
+	}
+    }
 }
 
 sub cpiostderr {
