@@ -2,7 +2,28 @@
 
 #use POSIX; &ENOENT;
 sub ENOENT { 2; }
-# Sorry about this, but POSIX.pm isn't necessarily available
+# Sorry about this, but the errno-part of POSIX.pm isn't in perl-*-base
+
+# Global variables:
+#  $alink            Alternative we are managing (ie the symlink we're making/removing) (install only)
+#  $name             Name of the alternative (the symlink) we are processing
+#  $apath            Path of alternative we are offering            
+#  $apriority        Priority of link (only when we are installing an alternative)
+#  $mode             action to perform (display / install / remove / display / auto / config)
+#  $manual           update-mode for alternative (manual / auto)
+#  $state            State of alternative:
+#                       expected: alternative with highest priority is the active alternative
+#                       expected-inprogress: busy selecting alternative with highest priority
+#                       unexpected: alternative another alternative is active / error during readlink
+#                       nonexistant: alternative-symlink does not exist
+#  $link             Link we are working with
+#  @slavenames       List with names of slavelinks
+#  %slavenum         Map from name of slavelink to slave-index (into @slavelinks)
+#  @slavelinks       List of slavelinks (indexed by slave-index)
+#  %versionnum       Map from currently available versions into @versions and @priorities
+#  @versions         List of available versions for alternative
+#  %priorities       Map from @version-index to priority
+#  %slavepath        Map from (@version-index,slavename) to slave-path
 
 $version= '0.93.80'; # This line modified by Makefile
 sub usageversion {
@@ -16,6 +37,7 @@ Usage: update-alternatives --install <link> <name> <path> <priority>
        update-alternatives --remove <name> <path>
        update-alternatives --auto <name>
        update-alternatives --display <name>
+       update-alternatives --config <name>
 <name> is the name in /etc/alternatives.
 <path> is the name referred to.
 <link> is the link pointing to /etc/alternatives/<name>.
@@ -66,7 +88,7 @@ while (@ARGV) {
         @ARGV >= 2 || &badusage("--remove needs <name> <path>");
         ($name,$apath,@ARGV) = @ARGV;
         $mode= 'remove';
-    } elsif (m/^--(display|auto)$/) {
+    } elsif (m/^--(display|auto|config)$/) {
         &checkmanymodes;
         @ARGV || &badusage("--$1 needs <name>");
         $mode= $1;
@@ -92,7 +114,7 @@ while (@ARGV) {
 defined($aslavelink{$name}) && &badusage("name $name is both primary and slave");
 $aslavelinkcount{$alink} && &badusage("link $link is both primary and slave");
 
-$mode || &badusage("need --display, --install, --remove or --auto");
+$mode || &badusage("need --display, --config, --install, --remove or --auto");
 $mode eq 'install' || !%slavelink || &badusage("--slave only allowed with --install");
 
 if (open(AF,"$admindir/$name")) {
@@ -121,7 +143,8 @@ if (open(AF,"$admindir/$name")) {
            }
        } else {
            # File not found - remove
-            &pr("Alternative for $name points to $version - which wasn't found.  Removing from list of alternatives.");
+           &pr("Alternative for $name points to $version - which wasn't found.  Removing from list of alternatives.")
+             if $verbosemode > 0;
            &gl("priority");
            for ($j=0; $j<=$#slavenames; $j++) {
                &gl("spath");
@@ -173,6 +196,15 @@ for ($i=0; $i<=$#versions; $i++) {
     }
 }
 
+if ($mode eq 'config') {
+    if (!$dataread) {
+	&pr("No alternatives for $name.");
+    } else {
+	&config_alternatives($name);
+	exit 0;
+    }
+}
+
 if (defined($linkname= readlink("$altdir/$name"))) {
     if ($linkname eq $best) {
         $state= 'expected';
@@ -194,7 +226,8 @@ if (defined($linkname= readlink("$altdir/$name"))) {
 # all independent
 
 if ($mode eq 'auto') {
-    &pr("Setting up automatic selection of $name.");
+    &pr("Setting up automatic selection of $name.")
+      if $verbosemode > 0;
     unlink("$altdir/$name.dpkg-tmp") || $! == &ENOENT ||
         &quit("unable to remove $altdir/$name.dpkg-tmp: $!");
     unlink("$altdir/$name") || $! == &ENOENT ||
@@ -203,7 +236,8 @@ if ($mode eq 'auto') {
     $manual= 'auto';
 } elsif ($state eq 'nonexistent') {
     if ($mode eq 'manual') {
-        &pr("$altdir/$name has been deleted, returning to automatic selection.");
+        &pr("$altdir/$name has been deleted, returning to automatic selection.")
+          if $verbosemode > 0;
         $mode= 'auto';
     }
 }
@@ -215,7 +249,8 @@ if ($mode eq 'auto') {
 
 if ($state eq 'unexpected' && $manual eq 'auto') {
     &pr("$altdir/$name has been changed (manually or by a script).\n".
-        "Switching to manual updates only.");
+        "Switching to manual updates only.")
+      if $verbosemode > 0;
     $manual= 'manual';
 }
 
@@ -226,12 +261,14 @@ if ($state eq 'unexpected' && $manual eq 'auto') {
 # state=unexpected => manual=manual
 
 &pr("Checking available versions of $name, updating links in $altdir ...\n".
-    "(You may modify the symlinks there yourself if desired - see \`man ln'.)");
+    "(You may modify the symlinks there yourself if desired - see \`man ln'.)")
+  if $verbosemode > 0;
 
 if ($mode eq 'install') {
     if ($link ne $alink && $link ne '') {
-        &pr("Renaming $name link from $link to $alink.");
-        rename($link,$alink) || $! == &ENOENT ||
+        &pr("Renaming $name link from $link to $alink.")
+          if $verbosemode > 0;
+        rename_mv($link,$alink) || $! == &ENOENT ||
             &quit("unable to rename $link to $alink: $!");
     }
     $link= $alink;
@@ -251,8 +288,9 @@ if ($mode eq 'install') {
         $slavelinkcount{$newslavelink}++ &&
             &quit("slave link name $newslavelink duplicated");
         if ($newslavelink ne $oldslavelink && $oldslavelink ne '') {
-            &pr("Renaming $sname slave link from $oldslavelink to $newslavelink.");
-            rename($oldslavelink,$newslavelink) || $! == &ENOENT ||
+            &pr("Renaming $sname slave link from $oldslavelink to $newslavelink.")
+              if $verbosemode > 0;
+            rename_mv($oldslavelink,$newslavelink) || $! == &ENOENT ||
                 &quit("unable to rename $oldslavelink to $newslavelink: $!");
         }
         $slavelinks[$j]= $newslavelink;
@@ -263,6 +301,10 @@ if ($mode eq 'install') {
 }
 
 if ($mode eq 'remove') {
+    if ($manual eq "manual" and $state = "expected") {
+    	&pr("Removing manually selected alternative - switching to auto mode");
+	$manual= "auto";
+    }
     if (defined($i= $versionnum{$apath})) {
         $k= $#versions;
         $versionnum{$versions[$k]}= $i;
@@ -274,7 +316,8 @@ if ($mode eq 'remove') {
             delete $slavepath{$k,$j};
         }
     } else {
-        &pr("Alternative $apath for $name not registered, not removing.");
+        &pr("Alternative $apath for $name not registered, not removing.")
+          if $verbosemode > 0;
     }
 }
 
@@ -283,7 +326,8 @@ for ($j=0; $j<=$#slavenames; $j++) {
         last if $slavepath{$i,$j} ne '';
     }
     if ($i > $#versions) {
-        &pr("Discarding obsolete slave link $slavenames[$j] ($slavelinks[$j]).");
+        &pr("Discarding obsolete slave link $slavenames[$j] ($slavelinks[$j]).")
+          if $verbosemode > 0;
         unlink("$altdir/$slavenames[$j]") || $! == &ENOENT ||
             &quit("unable to remove $slavenames[$j]: $!");
         unlink($slavelinks[$j]) || $! == &ENOENT ||
@@ -303,12 +347,14 @@ for ($j=0; $j<=$#slavenames; $j++) {
 }
         
 if ($manual eq 'manual') {
-    &pr("Automatic updates of $altdir/$name are disabled, leaving it alone.");
-    &pr("To return to automatic updates use \`update-alternatives --auto $name'.");
+    &pr("Automatic updates of $altdir/$name are disabled, leaving it alone.")
+      if $verbosemode > 0;
+    &pr("To return to automatic updates use \`update-alternatives --auto $name'.")
+      if $verbosemode > 0;
 } else {
     if ($state eq 'expected-inprogress') {
         &pr("Recovering from previous failed update of $name ...");
-        rename("$altdir/$name.dpkg-tmp","$altdir/$name") ||
+        rename_mv("$altdir/$name.dpkg-tmp","$altdir/$name") ||
             &quit("unable to rename $altdir/$name.dpkg-tmp to $altdir/$name: $!");
         $state= 'expected';
     }
@@ -346,7 +392,8 @@ close(AF) || &quit("unable to close $admindir/$name.dpkg-new: $!");
 
 if ($manual eq 'auto') {
     if ($best eq '') {
-        &pr("Last package providing $name ($link) removed, deleting it.");
+        &pr("Last package providing $name ($link) removed, deleting it.")
+          if $verbosemode > 0;
         unlink("$altdir/$name") || $! == &ENOENT ||
             &quit("unable to remove $altdir/$name: $!");
         unlink("$link") || $! == &ENOENT ||
@@ -359,19 +406,22 @@ if ($manual eq 'auto') {
     } else {
         if (!defined($linkname= readlink($link)) && $! != &ENOENT) {
             &pr("warning: $link is supposed to be a symlink to $altdir/$name\n".
-                " (or nonexistent); however, readlink failed: $!");
+                " (or nonexistent); however, readlink failed: $!")
+              if $verbosemode > 0;
         } elsif ($linkname ne "$altdir/$name") {
             unlink("$link.dpkg-tmp") || $! == &ENOENT ||
                 &quit("unable to ensure $link.dpkg-tmp nonexistent: $!");
             symlink("$altdir/$name","$link.dpkg-tmp") ||
                 &quit("unable to make $link.dpkg-tmp a symlink to $altdir/$name: $!");
-            rename("$link.dpkg-tmp",$link) ||
+            rename_mv("$link.dpkg-tmp",$link) ||
                 &quit("unable to install $link.dpkg-tmp as $link: $!");
         }
         if (defined($linkname= readlink("$altdir/$name")) && $linkname eq $best) {
-            &pr("Leaving $name ($link) pointing to $best.");
+            &pr("Leaving $name ($link) pointing to $best.")
+              if $verbosemode > 0;
         } else {
-            &pr("Updating $name ($link) to point to $best.");
+            &pr("Updating $name ($link) to point to $best.")
+              if $verbosemode > 0;
         }
         unlink("$altdir/$name.dpkg-tmp") || $! == &ENOENT ||
             &quit("unable to ensure $altdir/$name.dpkg-tmp nonexistent: $!");
@@ -379,47 +429,90 @@ if ($manual eq 'auto') {
     }
 }
 
-rename("$admindir/$name.dpkg-new","$admindir/$name") ||
+rename_mv("$admindir/$name.dpkg-new","$admindir/$name") ||
     &quit("unable to rename $admindir/$name.dpkg-new to $admindir/$name: $!");
 
 if ($manual eq 'auto') {
-    rename("$altdir/$name.dpkg-tmp","$altdir/$name") ||
+    rename_mv("$altdir/$name.dpkg-tmp","$altdir/$name") ||
         &quit("unable to install $altdir/$name.dpkg-tmp as $altdir/$name");
     for ($j=0; $j<=$#slavenames; $j++) {
         $sname= $slavenames[$j];
         $slink= $slavelinks[$j];
         if (!defined($linkname= readlink($slink)) && $! != &ENOENT) {
             &pr("warning: $slink is supposed to be a slave symlink to\n".
-                " $altdir/$sname, or nonexistent; however, readlink failed: $!");
+                " $altdir/$sname, or nonexistent; however, readlink failed: $!")
+              if $verbosemode > 0;
         } elsif ($linkname ne "$altdir/$sname") {
             unlink("$slink.dpkg-tmp") || $! == &ENOENT ||
                 &quit("unable to ensure $slink.dpkg-tmp nonexistent: $!");
             symlink("$altdir/$sname","$slink.dpkg-tmp") ||
                 &quit("unable to make $slink.dpkg-tmp a symlink to $altdir/$sname: $!");
-            rename("$slink.dpkg-tmp",$slink) ||
+            rename_mv("$slink.dpkg-tmp",$slink) ||
                 &quit("unable to install $slink.dpkg-tmp as $slink: $!");
         }
         $spath= $slavepath{$bestnum,$j};
         unlink("$altdir/$sname.dpkg-tmp") || $! == &ENOENT ||
             &quit("unable to ensure $altdir/$sname.dpkg-tmp nonexistent: $!");
         if ($spath eq '') {
-            &pr("Removing $sname ($slink), not appropriate with $best.");
+            &pr("Removing $sname ($slink), not appropriate with $best.")
+              if $verbosemode > 0;
             unlink("$altdir/$sname") || $! == &ENOENT ||
                 &quit("unable to remove $altdir/$sname: $!");
+	    unlink("$slink") || $! == &ENOENT ||
+	        &quit("unable to remove $slink: $!");
         } else {
             if (defined($linkname= readlink("$altdir/$sname")) && $linkname eq $spath) {
-                &pr("Leaving $sname ($slink) pointing to $spath.");
+                &pr("Leaving $sname ($slink) pointing to $spath.")
+                  if $verbosemode > 0;
             } else {
-                &pr("Updating $sname ($slink) to point to $spath.");
+                &pr("Updating $sname ($slink) to point to $spath.")
+                  if $verbosemode > 0;
             }
             symlink("$spath","$altdir/$sname.dpkg-tmp") ||
                 &quit("unable to make $altdir/$sname.dpkg-tmp a symlink to $spath: $!");
-            rename("$altdir/$sname.dpkg-tmp","$altdir/$sname") ||
+            rename_mv("$altdir/$sname.dpkg-tmp","$altdir/$sname") ||
                 &quit("unable to install $altdir/$sname.dpkg-tmp as $altdir/$sname: $!");
         }
     }
 }
 
+sub config_message {
+    if ($#versions == 0) {
+	print "\nThere is only 1 program which provides $name\n";
+	print "($versions[0]). Nothing to configure.\n";
+	return;
+    }
+    printf(STDOUT "\nThere are %s programs which provide \`$name'.\n\n", $#versions+1);
+    printf(STDOUT "  Selection    Command\n");
+    printf(STDOUT "-----------------------------------------------\n");
+    for ($i=0; $i<=$#versions; $i++) {
+	printf(STDOUT "%s%s    %s        %s\n", 
+	    (readlink("$altdir/$name") eq $versions[$i]) ? '*' : ' ',
+	    ($best eq $versions[$i]) ? '+' : ' ',
+	    $i+1, $versions[$i]);
+    }
+    printf(STDOUT "\nEnter to keep the default[*], or type selection number: ");
+}
+
+sub config_alternatives {
+    do {
+	&config_message;
+	if ($#versions == 0) { return; }
+	$preferred=<STDIN>;
+	chop($preferred);
+    } until $preferred eq '' || $preferred>=1 && $preferred<=$#versions+1 &&
+	($preferred =~ m/[0-9]*/);
+    if ($preferred ne '') {
+	$preferred--;
+	print STDOUT "Using \`$versions[$preferred]' to provide \`$name'.\n";
+	my $spath = $versions[$preferred];
+	symlink("$spath","$altdir/$name.dpkg-tmp") ||
+	    &quit("unable to make $altdir/$name.dpkg-tmp a symlink to $spath: $!");
+	rename_mv("$altdir/$name.dpkg-tmp","$altdir/$name") ||
+	    &quit("unable to install $altdir/$name.dpkg-tmp as $altdir/$name: $!");
+    }
+}
+
 sub pr { print(STDOUT "@_\n") || &quit("error writing stdout: $!"); }
 sub paf {
     $_[0] =~ m/\n/ && &quit("newlines prohibited in update-alternatives files ($_[0])");
@@ -434,5 +527,9 @@ sub gl {
 sub badfmt {
     &quit("internal error: $admindir/$name corrupt: $_[0]");
 }
-
+sub rename_mv {
+    return (rename($_[0], $_[1]) || (system(("mv", $_[0], $_[1])) == 0));
+}
 exit(0);
+
+# vim: nowrap ts=8 sw=4
